/* 
 * File:   AccesBdd.cpp
 * Author: Philippe Cruchet
 * 
 * Created on 22 avril 2025, 17:10
 */

#include "AccesBdd.h"

AccesBdd::AccesBdd(const char* dbPath, bool debug) : debugMode(debug), db(nullptr)
{
    if (!SPIFFS.begin(true))
    {
        throw BddException("Impossible de monter SPIFFS.");
    }

    String path = String("/spiffs/") + dbPath;
    if (sqlite3_open(path.c_str(), &db) != SQLITE_OK)
    {
        throw BddException("Impossible d'ouvrir la base de données.");
    }

    if (debugMode)
    {
        Serial.printf("[INFO] Total SPIFFS: %d bytes\n", SPIFFS.totalBytes());
        Serial.printf("[INFO] Used SPIFFS : %d bytes\n", SPIFFS.usedBytes());
        Serial.printf("[INFO] Base SQLite ouverte avec succès : %s\n", path.c_str());
    }
}

AccesBdd::~AccesBdd()
{
    if (db)
    {
        sqlite3_close(db);
    }
}

bool AccesBdd::executerCommande(const char* requete, std::function<void(sqlite3_stmt*) > binder)
{
    sqlite3_stmt* stmt = nullptr;
    bool success = false;

    if (sqlite3_prepare_v2(db, requete, -1, &stmt, nullptr) != SQLITE_OK)
    {
        throw BddException("Préparation commande SQL échouée : " + String(sqlite3_errmsg(db)));
    }

    binder(stmt);

    if (sqlite3_step(stmt) != SQLITE_DONE)
    {
        sqlite3_finalize(stmt);
        throw BddException("Exécution commande SQL échouée : " + String(sqlite3_errmsg(db)));
    }

    success = true;
    sqlite3_finalize(stmt);
    return success;
}

bool AccesBdd::ChangerMode(int id, char mode)
{
    const char* requete = "UPDATE Electrovannes SET mode = ? WHERE id = ?;";
    bool result = false;

    try
    {
        result = executerCommande(requete, [ = ](sqlite3_stmt * stmt){
                                  sqlite3_bind_text(stmt, 1, &mode, 1, SQLITE_STATIC);
                                  sqlite3_bind_int(stmt, 2, id);
        });
    }
    catch (const BddException& e)
    {
        Serial.println(e.what());
    }

    return result;
}

bool AccesBdd::ouvrirVanne(int id)
{
    const char* requete = "UPDATE ModeManuel SET etat = 'O' WHERE electrovanne_id = ?";
    bool result = false;

    try
    {
        result = executerCommande(requete, [ = ](sqlite3_stmt * stmt){
                                  sqlite3_bind_int(stmt, 1, id);
        });
    }
    catch (const BddException& e)
    {
        Serial.println(e.what());
    }

    return result;
}

bool AccesBdd::fermerVanne(int id)
{
    const char* requete = "UPDATE ModeManuel SET etat = 'F' WHERE electrovanne_id = ?";
    bool result = false;

    try
    {
        result = executerCommande(requete, [ = ](sqlite3_stmt * stmt){
                                  sqlite3_bind_int(stmt, 1, id);
        });
    }
    catch (const BddException& e)
    {
        Serial.println(e.what());
    }

    return result;
}

bool AccesBdd::modifierProgramme(int id, const String& dateHeureDebut, int duree, int frequence)
{
    sqlite3_stmt* stmt = nullptr;
    bool result = false;
    int exists = 0;

    try
    {
        const char* sqlSelect = "SELECT COUNT(*) FROM ModeProgramme WHERE electrovanne_id = ?";
        if (sqlite3_prepare_v2(db, sqlSelect, -1, &stmt, nullptr) != SQLITE_OK)
        {
            throw BddException("Erreur préparation SELECT programme : " + String(sqlite3_errmsg(db)));
        }
        sqlite3_bind_int(stmt, 1, id);
        if (sqlite3_step(stmt) == SQLITE_ROW)
        {
            exists = sqlite3_column_int(stmt, 0);
        }
        sqlite3_finalize(stmt);

        const char* sqlUpdate = "UPDATE ModeProgramme SET date_heure_debut = ?, duree = ?, frequence = ? WHERE electrovanne_id = ?";
        const char* sqlInsert = "INSERT INTO ModeProgramme (electrovanne_id, date_heure_debut, duree, frequence) VALUES (?, ?, ?, ?)";
        const char* sqlToUse = exists ? sqlUpdate : sqlInsert;

        result = executerCommande(sqlToUse, [ = ](sqlite3_stmt * stmt){
                                  if (exists)
            {
                                  sqlite3_bind_text(stmt, 1, dateHeureDebut.c_str(), -1, SQLITE_TRANSIENT);
                                  sqlite3_bind_int(stmt, 2, duree);
                                  sqlite3_bind_int(stmt, 3, frequence);
                                  sqlite3_bind_int(stmt, 4, id);
            }
            else
            {
                                  sqlite3_bind_int(stmt, 1, id);
                                  sqlite3_bind_text(stmt, 2, dateHeureDebut.c_str(), -1, SQLITE_TRANSIENT);
                                  sqlite3_bind_int(stmt, 3, duree);
                                  sqlite3_bind_int(stmt, 4, frequence);
            }
        });
    }
    catch (const BddException& e)
    {
        Serial.println(e.what());
    }

    return result;
}

bool AccesBdd::modifierAutomatique(int id, int hygroMin, int hygroMax)
{
    sqlite3_stmt* stmt = nullptr;
    bool result = false;
    int exists = 0;

    try
    {
        const char* sqlSelect = "SELECT COUNT(*) FROM ModeAutomatique WHERE electrovanne_id = ?";
        if (sqlite3_prepare_v2(db, sqlSelect, -1, &stmt, nullptr) != SQLITE_OK)
        {
            throw BddException("Erreur préparation SELECT automatique : " + String(sqlite3_errmsg(db)));
        }
        sqlite3_bind_int(stmt, 1, id);
        if (sqlite3_step(stmt) == SQLITE_ROW)
        {
            exists = sqlite3_column_int(stmt, 0);
        }
        sqlite3_finalize(stmt);

        const char* sqlUpdate = "UPDATE ModeAutomatique SET hygro_min = ?, hygro_max = ? WHERE electrovanne_id = ?";
        const char* sqlInsert = "INSERT INTO ModeAutomatique (electrovanne_id, hygro_min, hygro_max) VALUES (?, ?, ?)";
        const char* sqlToUse = exists ? sqlUpdate : sqlInsert;

        result = executerCommande(sqlToUse, [ = ](sqlite3_stmt * stmt){
                                  if (exists)
            {
                                  sqlite3_bind_int(stmt, 1, hygroMin);
                                  sqlite3_bind_int(stmt, 2, hygroMax);
                                  sqlite3_bind_int(stmt, 3, id);
            }
            else
            {
                                  sqlite3_bind_int(stmt, 1, id);
                                  sqlite3_bind_int(stmt, 2, hygroMin);
                                  sqlite3_bind_int(stmt, 3, hygroMax);
            }
        });
    }
    catch (const BddException& e)
    {
        Serial.println(e.what());
    }

    return result;
}

void AccesBdd::afficherVannes()
{
    sqlite3_stmt* stmt = nullptr;
    if (db)
    {
        try
        {
            const char* sqlSelect = "SELECT id, nom, mode FROM Electrovannes;";
            if (sqlite3_prepare_v2(db, sqlSelect, -1, &stmt, nullptr) != SQLITE_OK)
            {
                throw BddException("Erreur préparation SELECT programme : " + String(sqlite3_errmsg(db)));
            }

            while (sqlite3_step(stmt) == SQLITE_ROW)
            {
                int id = sqlite3_column_int(stmt, 0);
                const char* nom = reinterpret_cast<const char*> (sqlite3_column_text(stmt, 1));
                char mode = sqlite3_column_text(stmt, 2)[0];

                Serial.print(F("Vanne : "));
                Serial.print(nom);
                Serial.print(F(" (id "));
                Serial.print(id);
                Serial.print(F(") - Mode : "));

                switch (mode)
                {
                case 'M':
                    Serial.println(F("Manuel"));
                    afficherEtatManuel(id);
                    break;
                case 'P':
                    Serial.println(F("Programmé"));
                    afficherProgramme(id);
                    break;
                case 'A':
                    Serial.println(F("Automatique"));
                    afficherAuto(id);
                    break;
                default:
                    Serial.println(F("Inconnu"));
                    break;
                }

                Serial.println(F("----------------------------"));
            }

            sqlite3_finalize(stmt);
        }
        catch (const BddException& e)
        {
            Serial.println(e.what());
        }
    }
}

void AccesBdd::afficherEtatManuel(int id) {
    const char* requete = "SELECT etat FROM ModeManuel WHERE electrovanne_id = ?;";

    bool success = executerCommande(requete, [=](sqlite3_stmt* stmt) {
        sqlite3_bind_int(stmt, 1, id);

        int rc = sqlite3_step(stmt);
        if (rc == SQLITE_ROW) {
            char etat = sqlite3_column_text(stmt, 0)[0];
            Serial.print(F("  État : "));
            Serial.println(etat == 'O' ? F("Ouverte") : F("Fermée"));
        } else if (rc == SQLITE_DONE) {
            Serial.println(F("  Aucun état manuel enregistré pour cette vanne."));
        } else {
            throw BddException(F("Erreur lors de la lecture de l'état manuel."));
        }
    });

    if (!success) {
        throw BddException(F("Échec de la commande SQL dans afficherEtatManuel."));
    }
}

void AccesBdd::afficherProgramme(int id)
{
    const char* requete = "SELECT date_heure_debut, duree, frequence FROM ModeProgramme WHERE electrovanne_id = ?;";
    try
    {
        bool success = executerCommande(requete, [id](sqlite3_stmt* stmt)
        {
            sqlite3_bind_int(stmt, 1, id);
            if (sqlite3_step(stmt) == SQLITE_ROW)
            {
                const char* debut = reinterpret_cast<const char*>(sqlite3_column_text(stmt, 0));
                int duree = sqlite3_column_int(stmt, 1);
                int frequence = sqlite3_column_int(stmt, 2);

                Serial.print(F("  Début : "));
                Serial.println(debut);
                Serial.print(F("  Durée : "));
                Serial.print(duree);
                Serial.println(F(" min"));
                Serial.print(F("  Fréquence : toutes les "));
                Serial.print(frequence);
                Serial.println(F(" heures"));
            }
        });

        if (!success)
        {
            throw BddException(F("Erreur lors de la récupération des données de programmation"));
        }
    }
    catch (const BddException& e)
    {
        if (debugMode)
        {
            Serial.print(F("[ERREUR] "));
            Serial.println(e.what());
        }
    }
}

void AccesBdd::afficherAuto(int id)
{
    const char* requete = "SELECT hygro_min, hygro_max FROM ModeAutomatique WHERE electrovanne_id = ?;";
    try
    {
        bool success = executerCommande(requete, [id](sqlite3_stmt* stmt)
        {
            sqlite3_bind_int(stmt, 1, id);
            if (sqlite3_step(stmt) == SQLITE_ROW)
            {
                float hygro_min = sqlite3_column_double(stmt, 0);
                float hygro_max = sqlite3_column_double(stmt, 1);

                Serial.print(F("  Hygrométrie min : "));
                Serial.print(hygro_min);
                Serial.println(F(" %"));
                Serial.print(F("  Hygrométrie max : "));
                Serial.print(hygro_max);
                Serial.println(F(" %"));
            }
        });

        if (!success)
        {
            throw BddException(F("Erreur lors de la récupération des seuils automatiques"));
        }
    }
    catch (const BddException& e)
    {
        if (debugMode)
        {
            Serial.print(F("[ERREUR] "));
            Serial.println(e.what());
        }
    }
}




