/** 
 * @File   AccesBdd.h
 * @Author Philippe Cruchet
 *
 * @Created on 22 avril 2025, 17:10
 */

#ifndef ACCES_BDD_H
#define ACCES_BDD_H

#include <Arduino.h>
#include <SPIFFS.h>
#include <sqlite3.h>
#include "BddException.h"

/**
 * @brief Classe de gestion de la base SQLite pour le pilotage des électrovannes.
 */
class AccesBdd {
public:
    /**
     * @brief Constructeur de la classe.
     * @param dbPath Chemin relatif du fichier SQLite (dans SPIFFS).
     * @param debugModeActive Active ou non les messages de débogage.
     */
    AccesBdd(const char* dbPath, bool debugModeActive = false);

    /**
     * @brief Destructeur : ferme la base si ouverte.
     */
    ~AccesBdd();

    /**
     * @brief Exécute une commande SQL avec liaison de paramètres.
     * @param requete La requête SQL à exécuter.
     * @param binder Fonction lambda pour lier les paramètres à la requête.
     * @return true si l'exécution a réussi, false sinon.
     */
    bool executerCommande(const char* requete, std::function<void(sqlite3_stmt*) > binder);

    /**
     * @brief Change le mode de fonctionnement d'une vanne.
     * @param id Identifiant de la vanne.
     * @param mode Nouveau mode ('M', 'P' ou 'A').
     * @return true si le changement a réussi, false sinon.
     */
    bool ChangerMode(int id, char mode);

    /**
     * @brief Ouvre une vanne (état 'O').
     * @param id Identifiant de la vanne.
     * @return true si l'opération réussit, false sinon.
     */
    bool ouvrirVanne(int id);

    /**
     * @brief Ferme une vanne (état 'F').
     * @param id Identifiant de la vanne.
     * @return true si l'opération réussit, false sinon.
     */
    bool fermerVanne(int id);

    /**
     * @brief Modifie ou insère une configuration automatique pour une électrovanne.
     * @param id Identifiant de l'électrovanne.
     * @param hygroMin Valeur minimale d'hygrométrie pour ouverture.
     * @param hygroMax Valeur maximale d'hygrométrie pour fermeture.
     * @return true si l'opération s'est bien déroulée, false sinon.
     */
    bool modifierAutomatique(int id, int hygroMin, int hygroMax);

    /**
     * @brief Modifie ou insère une programmation pour une électrovanne.
     * @param id Identifiant de l'électrovanne.
     * @param dateHeureDebut Date et heure de début (format ISO : "DD-MM-YYYY HH:MM:SS").
     * @param duree Durée d'arrosage en minutes.
     * @param frequence Fréquence d'arrosage en jours.
     * @return true si l'opération a réussi, false sinon.
     */
    bool modifierProgramme(int id, const String& dateHeureDebut, int duree, int frequence);

    /**
     * @brief Affiche les informations de toutes les vannes dans le port série.
     */
    void afficherVannes();

    /**
     * @brief Affiche l'état manuel (ouverte/fermée) d'une vanne.
     * @param id Identifiant de la vanne.
     */
    void afficherEtatManuel(int id);

    /**
     * @brief Affiche les informations de programmation d'une vanne.
     * @param id Identifiant de la vanne.
     */
    void afficherProgramme(int id);

    /**
     * @brief Affiche les seuils d'hygrométrie pour le mode automatique d'une vanne.
     * @param id Identifiant de la vanne.
     */
    void afficherAuto(int id);

 
private:
    sqlite3* db; ///< Pointeur vers la base de données SQLite.
    bool debugMode; ///< Mode débogage actif ou non.

    /**
     * @brief Ouvre la base de données SQLite.
     * @param dbPath Chemin relatif du fichier SQLite.
     * @return true si la base est ouverte avec succès, false sinon.
     */
    bool ouvrirBase(const char* dbPath);
};

#endif /*ACCES_BDD_H*/
